@@ -161,50 +161,114 @@ int get_mmc_profile (int fd)
 {
   Scsi_Command cmd(fd);
   int retval = -1;
+  unsigned char formats[260],disc_info[32];
   unsigned char page[20];
+  unsigned char buf[8],inq[128];
   unsigned char *sense=cmd.sense();
   int i;
+  int profile=0,once=1,blank=0,err;
+  unsigned int len;
 
   /* For valgrind */
   memset (&page, 0, sizeof (page));
+  memset (&buf, 0, sizeof (buf));
+  memset (&inq, 0, sizeof (inq));
+  memset (&disc_info, 0, sizeof (disc_info));
+  memset (&formats, 0, sizeof (formats));
 
-  cmd[0]=0x46;
-  cmd[1]=2;
-  cmd[8]=8;
-  cmd[9]=0;
-  if (cmd.transport(READ,page,8))
-  {    /* if (sense[0]==0) perror("Unable to ioctl");
-	  else             fprintf(stderr,"GET CONFIGURATION failed with "
-			  "SK=%xh/ASC=%02xh/ASCQ=%02xh\n",
-			  sense[2]&0xF,sense[12],sense[13]); */
-	  goto bail;
+  // INQUIRY is considered to be "non-intrusive" in a sense that
+  // it won't interfere with any other operation nor clear sense
+  // data, which might be important to retain for security reasons.
+
+  cmd[0] = 0x12;	// INQUIRY
+  cmd[4] = 36;
+  cmd[5] = 0;
+  if ((err=cmd.transport(READ,inq,36))) {
+    /*sperror ("INQUIRY",err);*/
+    goto bail;
   }
 
-  // See if it's 2 gen drive by checking if DVD+R profile is an option
-  {
-    int len=4+(page[0]<<24|page[1]<<16|page[2]<<8|page[3]);
-    if (len>264)
-    {
-      /* fprintf (stderr,"insane profile list length [%d].\n",len); */
-      goto bail;
-    }
-    unsigned char *list=new unsigned char[len];
-
-    cmd[0]=0x46;
-    cmd[1]=2;
-    cmd[7]=len>>8;
-    cmd[8]=len;
-    cmd[9]=0;
-    if (cmd.transport(READ,list,len))
-    {
-      /* fprintf(stderr,"GET CONFIGURATION failed with "
-		      "SK=%xh/ASC=%02xh/ASCQ=%02xh\n",
-		      sense[2]&0xF,sense[12],sense[13]); */
-      goto bail;
-    }
+  // make sure we're talking to MMC unit, for security reasons...
+  if ((inq[0]&0x1F) != 5) {
+    /*fprintf (stderr,":-( not an MMC unit!\n");*/
+    goto bail;
   }
 
-  return page[6]<<8|page[7];
+  do {
+    cmd[0] = 0x46;
+    cmd[8] = sizeof(buf);
+    cmd[9] = 0;
+    if ((err=cmd.transport(READ,buf,sizeof(buf)))) {
+      /*sperror ("GET CONFIGURATION",err);*/
+      /*fprintf (stderr,":-( non-MMC unit?\n");*/
+      goto bail;
+    }
+
+    if ((profile = buf[6]<<8|buf[7]) || !once) break;
+
+    // no media?
+    cmd[0] = 0;	// TEST UNIT READY
+    cmd[5] = 0;
+    if ((cmd.transport()&0xFFF00) != 0x23A00) break;
+
+    // try to load tray...
+    cmd[0] = 0x1B;	// START/STOP UNIT
+    cmd[4] = 0x3;	// "Load"
+    cmd[5] = 0;
+    if ((err=cmd.transport ())) {
+      /*sperror ("LOAD TRAY",err);*/
+      goto bail;
+    }
+
+    wait_for_unit (cmd);
+  } while (once--);
+
+  cmd[0] = 0x51;	// READ DISC INFORMATION
+  cmd[8] = sizeof(disc_info);
+  cmd[9] = 0;
+  if ((err=cmd.transport (READ,disc_info,sizeof(disc_info)))) {
+    /*sperror ("READ DISC INFORMATION",err);*/
+    goto bail;
+  }
+
+  // see if it's blank media
+  if ((disc_info[2]&3) == 0)	blank=0x10000;
+
+  if (profile != 0x1A && profile != 0x13 && profile != 0x12)
+    return blank|profile;
+
+  cmd[0] = 0x23;	// READ FORMAT CAPACITIES
+  cmd[8] = 12;
+  cmd[9] = 0;
+  if ((err=cmd.transport (READ,formats,12))) {
+    /*sperror ("READ FORMAT CAPACITIES",err);*/
+    goto bail;
+  }
+
+  len = formats[3];
+  if (len&7 || len<16) {
+    /*fprintf (stderr,":-( FORMAT allocaion length isn't sane");*/
+    goto bail;
+  }
+
+  cmd[0] = 0x23;	// READ FORMAT CAPACITIES
+  cmd[7] = (4+len)>>8;
+  cmd[8] = (4+len)&0xFF;
+  cmd[9] = 0;
+  if ((err=cmd.transport (READ,formats,4+len))) {
+    /*sperror ("READ FORMAT CAPACITIES",err);*/
+    goto bail;
+  }
+
+  if (len != formats[3]) {
+    /*fprintf (stderr,":-( parameter length inconsistency\n");*/
+    goto bail;
+  }
+
+  // see if it's not formatted
+  if ((formats[8]&3) != 2) blank = 0x10000;
+
+  return blank|profile;
 
 bail:
 
@@ -213,7 +277,7 @@ bail:
 }
 
 extern "C"
-int get_disc_status (int fd, int *empty, int *is_rewritable)
+int get_disc_status (int fd, int *empty, int *is_rewritable, int *is_blank)
 {
   Scsi_Command cmd(fd);
   int retval = -1;
@@ -223,6 +287,7 @@ int get_disc_status (int fd, int *empty, int *is_rewritable)
 
   *empty = 0;
   *is_rewritable = 0;
+  *is_blank = 0;
 
   /* For valgrind */
   memset (&page, 0, sizeof (page));
@@ -248,6 +313,8 @@ int get_disc_status (int fd, int *empty, int *is_rewritable)
   
   *is_rewritable = (page[2] & 0x10) != 0;
 
+  *is_blank = (page[2] & 3) == 0;
+
   return 0;
 
 bail:
@@ -305,11 +372,14 @@ static unsigned int get_2k_capacity (Scsi_Command &cmd, int mmc_profile)
   unsigned int	nwa,free_blocks;
   unsigned char formats[260];
   int		i,obligatory,len,err;
+  int           profile = 0;
   /* Wild guess */
   int           next_track = 1;
 
+  profile = mmc_profile & 0xFFFF;
+
 retry:
-  if (mmc_profile == 0x1A || mmc_profile == 0x13 || mmc_profile == 0x12) {
+  if (profile == 0x1A || profile == 0x14 || profile == 0x13 || profile == 0x12) {
     memset (&formats, 0, sizeof (formats));
     cmd[0] = 0x23;      // READ FORMAT CAPACITIES
     cmd[8] = 12;
@@ -349,6 +419,7 @@ retry:
     {	case 0x1A:		// DVD+RW
 	    obligatory=0x26;
 	case 0x13:		// DVD-RW Restricted Overwrite
+	case 0x14:		// DVD-RW Sequential
 	    for (i=8,len=formats[3];i<len;i+=8)
 		if ((formats [4+i+4]>>2) == obligatory) break;
 
@@ -379,7 +450,6 @@ retry:
 	    break;
 
 	case 0x11:		// DVD-R
-	case 0x14:		// DVD-RW Sequential
 	case 0x1B:		// DVD+R
 	case 0x2B:		// DVD+R Double Layer
 	    cmd[0] = 0x52;	// READ TRACK INFORMATION
